<?php

declare(strict_types=1);

namespace Pinpoint\Conesso\Api;

use Magento\Framework\Api\SearchCriteriaInterface;

interface ContactRepositoryInterface
{
    /**
     * Save contact
     * @param \Pinpoint\Conesso\Api\Data\ContactInterface $contact
     * @return \Pinpoint\Conesso\Api\Data\ContactInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function save(
        \Pinpoint\Conesso\Api\Data\ContactInterface $contact
    );

    /**
     * Retrieve contact
     * @param int $contactId
     * @return \Pinpoint\Conesso\Api\Data\ContactInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function get($contactId);


    /**
     * Retrieve contact by customer id
     * @param int $customerId
     * @return \Pinpoint\Conesso\Api\Data\ContactInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getByCustomerId($customerId);

    /**
     * Retrieve contacts matching the specified criteria.
     * @param \Magento\Framework\Api\SearchCriteriaInterface $searchCriteria
     * @return \Pinpoint\Conesso\Api\Data\ContactSearchResultsInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $searchCriteria
    );

    /**
     * Delete contact
     * @param \Pinpoint\Conesso\Api\Data\ContactInterface $contact
     * @return bool true on success
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function delete(
        \Pinpoint\Conesso\Api\Data\ContactInterface $contact
    );

    /**
     * Delete contact by ID
     * @param int $contactId
     * @return bool true on success
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function deleteById($contactId);
}

