<?php

namespace Pinpoint\Conesso\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\Stdlib\DateTime\DateTime;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Store\Model\ScopeInterface as ScopeInterfaceAlias;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\Conesso\Logger\Logger;
use Pinpoint\Conesso\Api\ConfigConstantsInterface;

class Data extends AbstractHelper
{
    /**
     * @var StoreManagerInterface
     */
    public $storeManager;

    /**
     * @var Store
     */
    public $store;

    /**
     * @var DateTime
     */
    public $datetime;

    /**
     * @var Json
     */
    public $serializer;

    /**
     * @var EncryptorInterface
     */
    public $encryptor;

    /**
     * @var TimezoneInterface
     */
    private $timezone;

    /**
     * @var Logger
     */
    private $logger;

    /**
     * @var RequestInterface
     */
    private $request;

    /**
     * @param Context $context
     * @param StoreManagerInterface $storeManager
     * @param Store $store
     * @param Json $serilizer
     * @param DateTime $dateTime
     * @param TimezoneInterface $timezone
     * @param EncryptorInterface $encryptor
     * @param Logger $logger
     * @param RequestInterface $request
     */
    public function __construct(
        Context               $context,
        StoreManagerInterface $storeManager,
        Store                 $store,
        Json                  $serilizer,
        DateTime              $dateTime,
        TimezoneInterface     $timezone,
        EncryptorInterface    $encryptor,
        Logger                $logger,
        RequestInterface      $request
    )
    {
        $this->serializer = $serilizer;
        $this->storeManager = $storeManager;
        $this->store = $store;
        $this->datetime = $dateTime;
        $this->timezone = $timezone;
        $this->encryptor = $encryptor;
        $this->logger = $logger;
        $this->request = $request;

        parent::__construct($context);
    }

    /**
     * @return boolean
     */
    public function isEnabled()
    {
        return $this->scopeConfig->isSetFlag(
            ConfigConstantsInterface::XML_PATH_CONESSO_MODULE_ENABLE,
            ScopeInterfaceAlias::SCOPE_STORE
        );
    }

    /**
     * Get api mode.
     *
     * @return string|boolean
     */
    public function getApiMode()
    {
        return $this->scopeConfig->getValue(
            ConfigConstantsInterface::XML_PATH_CONESSO_API_MODE,
            ScopeInterfaceAlias::SCOPE_STORE
        );
    }

    /**
     * Get all stores.
     *
     * @param boolean|false $default
     *
     * @return \Magento\Store\Api\Data\StoreInterface[]
     */
    public function getStores($default = false)
    {
        return $this->storeManager->getStores($default);
    }

    /**
     * @return string|boolean
     */
    public function getApiKeyProd()
    {
        $value = $this->scopeConfig->getValue(ConfigConstantsInterface::XML_PATH_CONESSO_API_KEY_PROD, ScopeInterfaceAlias::SCOPE_STORE);
        return $value ? $this->encryptor->decrypt($value) : '';
    }

    /**
     * @return string|boolean
     */
    public function getApiKeyTest()
    {
        $value = $this->scopeConfig->getValue(ConfigConstantsInterface::XML_PATH_CONESSO_API_KEY_TEST, ScopeInterfaceAlias::SCOPE_STORE);
        return $this->encryptor->decrypt($value);
    }

    /**
     * Get all websites.
     *
     * @param boolean|false $default
     *
     * @return \Magento\Store\Api\Data\WebsiteInterface[]
     */
    public function getWebsites($default = false)
    {
        return $this->storeManager->getWebsites($default);
    }

    /**
     * Get config scope value.
     *
     * @param string $path
     * @param string $contextScope
     * @param null $contextScopeId
     *
     * @return integer|float|string|boolean
     */
    public function getConfigValue(
        $path,
        $contextScope = 'default',
        $contextScopeId = null
    )
    {
        $config = $this->scopeConfig->getValue(
            $path,
            $contextScope,
            $contextScopeId
        );

        return $config;
    }

    /**
     * Get if the debug log is enabled
     *
     * @return boolean
     */
    public function isDebugEnabled()
    {
        return $this->scopeConfig->isSetFlag(ConfigConstantsInterface::XML_PATH_CONESSO_API_DEBUG_ENABLE, ScopeInterfaceAlias::SCOPE_STORE);
    }

    /**
     * Log data to the extension's log file.
     *
     * @param string $data
     * @param array $extra
     *
     * @return void
     */
    public function log($data, $extra = [])
    {
        $this->logger->info($data, $extra);
    }

    /**
     * Log data to the extension's log file.
     *
     * @param string $message
     * @param array $extra
     *
     * @return void
     */
    public function debug($message, $extra = [])
    {
        $this->logger->debug($message, $extra);
    }

    /**
     * Log data to the extension's log file.
     *
     * @param string $message
     * @param array $extra
     *
     * @return void
     */
    public function error($message, $extra = [])
    {
        $this->logger->error($message, $extra);
    }
}
