<?php

declare(strict_types=1);

namespace Pinpoint\Conesso\Model;

use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Model\Context;
use Magento\Framework\Registry;
use Pinpoint\Conesso\Api\Data\ContactInterface;
use Pinpoint\Conesso\Api\Data\ContactInterfaceFactory;
use Pinpoint\Conesso\Model\ResourceModel\Contact\Collection;

class Contact extends \Magento\Framework\Model\AbstractModel
{

    const ENTITY = 'conesso_contact';

    /**
     * @var ContactInterfaceFactory
     */
    protected $contactDataFactory;

    /**
     * @var DataObjectHelper
     */
    protected $dataObjectHelper;

    /**
     * @var string
     */
    protected $_eventPrefix = 'conesso_contact';

    /**
     * @param Context $context
     * @param Registry $registry
     * @param ContactInterfaceFactory $contactDataFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param ResourceModel\Contact $resource
     * @param Collection $resourceCollection
     * @param array $data
     */
    public function __construct(
        Context $context,
        Registry $registry,
        ContactInterfaceFactory $contactDataFactory,
        DataObjectHelper $dataObjectHelper,
        ResourceModel\Contact $resource,
        Collection $resourceCollection,
        array $data = []
    ) {
        $this->contactDataFactory = $contactDataFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * Retrieve Contact model with Contact data
     * @return ContactInterface
     */
    public function getDataModel()
    {
        $contactData = $this->getData();

        $contactDataObject = $this->contactDataFactory->create();
        $this->dataObjectHelper->populateWithArray(
            $contactDataObject,
            $contactData,
            ContactInterface::class
        );

        return $contactDataObject;
    }
}

