<?php

namespace Pinpoint\Conesso\Observer\Contact\Customer;

use Magento\Customer\Model\CustomerFactory;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Stdlib\DateTime;
use Magento\Newsletter\Model\SubscriberFactory;
use Pinpoint\Conesso\Helper\Data;
use Pinpoint\Conesso\Model\ContactFactory;
use Pinpoint\Conesso\Model\ContactRepository;
use Pinpoint\Conesso\Model\ResourceModel\Contact;

/**
 * Create/Update contact for conesso.
 */
class ContactUpdate implements ObserverInterface
{
    /**
     * @var Contact
     */
    private $contactResource;

    /**
     * @var Data
     */
    private $helper;

    /**
     * @var CustomerFactory
     */
    private $customerFactory;

    /**
     * @var ContactRepository
     */
    private $contactRepository;

    /**
     * @var ContactFactory
     */
    private $contactFactory;

    /**
     * @var SubscriberFactory
     */
    private $subscriberFactory;

    /**
     * @var DateTime
     */
    private $dateTime;

    /**
     * @var bool
     */
    private $isRunningFlag;

    /**
     *
     * @param ContactFactory $contactFactory
     * @param ContactRepository $contactRepository
     * @param CustomerFactory $customerFactory
     * @param Data $data
     * @param Contact $contactResource
     * @param SubscriberFactory $subscriberFactory
     * @param DateTime $dateTime
     */
    public function __construct(
        ContactFactory $contactFactory,
        ContactRepository $contactRepository,
        CustomerFactory $customerFactory,
        Data $data,
        Contact $contactResource,
        SubscriberFactory $subscriberFactory,
        DateTime $dateTime
    ) {
        $this->contactFactory = $contactFactory;
        $this->contactRepository = $contactRepository;
        $this->contactResource = $contactResource;
        $this->customerFactory = $customerFactory;
        $this->helper = $data;
        $this->subscriberFactory = $subscriberFactory;
        $this->dateTime = $dateTime;
    }

    /**
     *
     * @param Observer $observer
     *
     * @return $this
     */
    public function execute(Observer $observer)
    {
        //check if sync enabled
        if (!$this->helper->isEnabled() || $this->isRunningFlag) {
            return $this;
        }

        /*
         * @todo fill up the $contact for other columns
         */
        $customer = $observer->getEvent()->getCustomer();
        $websiteId  = $customer->getWebsiteId();
        $storeId = $customer->getStoreId();

        $this->isRunningFlag = true;

        $email = $customer->getEmail();
        $customerId = $customer->getEntityId();
        $subscriber = $this->subscriberFactory->create()
            ->loadByCustomer($customerId, $websiteId);
        $isSubscribed = $subscriber->isSubscribed();
        $emailBefore = '';

        try {
            $contact = $this->contactRepository->getByCustomerId($customerId);
            if ($contact->getContactId()) {
                $emailBefore = $contact->getEmail();
            }

            $emailAddress = empty($emailBefore) ? $email : $emailBefore;

            //email change detection
            if ($emailBefore && $email != $emailBefore) {
                // update email
                $contact->setEmail($email);
            } elseif (!$emailBefore) {
                //for new contacts update email
                $contact->setEmail($email);
            }

            $contact
                ->setStoreId($storeId)
                ->setWebsiteId($websiteId)
                ->setCustomerId($customerId);

            if ($isSubscribed) {
                $contact->setLastSubscribedAt($this->dateTime->formatDate(true));
            }

            $this->contactRepository->save($contact);
        } catch (\Exception $e) {
            $this->helper->debug((string)$e, []);
        }

        return $this;
    }
}
